@@ -9,15 +9,22 @@ Widget that handle communications between the IPython Console and
 the Variable Explorer
 """
 
+import time
+
 from qtpy.QtCore import QEventLoop
 from qtpy.QtWidgets import QMessageBox
 
-from ipykernel.pickleutil import CannedObject
-from ipykernel.serialize import deserialize_object
+import cloudpickle
 from qtconsole.rich_jupyter_widget import RichJupyterWidget
 
-from spyder.config.base import _
-from spyder.py3compat import to_text_string
+from spyder.config.base import _, debug_print
+from spyder.py3compat import PY2, to_text_string
+
+
+try:
+    time.monotonic  # time.monotonic new in 3.3
+except AttributeError:
+    time.monotonic = time.time
 
 
 class NamepaceBrowserWidget(RichJupyterWidget):
@@ -63,9 +70,12 @@ class NamepaceBrowserWidget(RichJupyterWidget):
 
     def set_namespace_view_settings(self):
         """Set the namespace view settings"""
-        settings = to_text_string(self.namespacebrowser.get_view_settings())
-        code = u"get_ipython().kernel.namespace_view_settings = %s" % settings
-        self.silent_execute(code)
+        if self.namespacebrowser:
+            settings = to_text_string(
+                self.namespacebrowser.get_view_settings())
+            code =(u"get_ipython().kernel.namespace_view_settings = %s" %
+                   settings)
+            self.silent_execute(code)
 
     def get_value(self, name):
         """Ask kernel for a value"""
@@ -98,7 +108,9 @@ class NamepaceBrowserWidget(RichJupyterWidget):
     def set_value(self, name, value):
         """Set value for a variable"""
         value = to_text_string(value)
-        code = u"get_ipython().kernel.set_value('%s', %s)" % (name, value)
+        code = u"get_ipython().kernel.set_value('%s', %s, %s)" % (name, value,
+                                                                  PY2)
+
         if self._reading:
             self.kernel_client.input(u'!' + code)
         else:
@@ -159,41 +171,37 @@ class NamepaceBrowserWidget(RichJupyterWidget):
         return self._kernel_reply
 
     # ---- Private API (defined by us) ------------------------------
-    def _handle_data_message(self, msg):
+    def _handle_spyder_msg(self, msg):
         """
-        Handle raw (serialized) data sent by the kernel
-
-        We only handle data asked by Spyder, in case people use
-        publish_data for other purposes.
+        Handle internal spyder messages
         """
-        # Deserialize data
-        try:
-            data = deserialize_object(msg['buffers'])[0]
-        except Exception as msg:
-            self._kernel_value = None
-            self._kernel_reply = repr(msg)
+        spyder_msg_type = msg['content'].get('spyder_msg_type')
+        if spyder_msg_type == 'data':
+            # Deserialize data
+            try:
+                if PY2:
+                    value = cloudpickle.loads(msg['buffers'][0])
+                else:
+                    value = cloudpickle.loads(bytes(msg['buffers'][0]))
+            except Exception as msg:
+                self._kernel_value = None
+                self._kernel_reply = repr(msg)
+            else:
+                self._kernel_value = value
             self.sig_got_reply.emit()
             return
-
-        # Receive values asked for Spyder
-        value = data.get('__spy_data__', None)
-        if value is not None:
-            if isinstance(value, CannedObject):
-                value = value.get_object()
-            self._kernel_value = value
-            self.sig_got_reply.emit()
-            return
-
-        # Receive Pdb state and dispatch it
-        pdb_state = data.get('__spy_pdb_state__', None)
-        if pdb_state is not None and isinstance(pdb_state, dict):
-            self.refresh_from_pdb(pdb_state)
-
-        # Run Pdb continue to get to the first breakpoint
-        # Fixes 2034
-        pdb_continue = data.get('__spy_pdb_continue__', None)
-        if pdb_continue:
+        elif spyder_msg_type == 'pdb_state':
+            pdb_state = msg['content']['pdb_state']
+            if pdb_state is not None and isinstance(pdb_state, dict):
+                self.refresh_from_pdb(pdb_state)
+        elif spyder_msg_type == 'pdb_continue':
+            # Run Pdb continue to get to the first breakpoint
+            # Fixes 2034
             self.write_to_stdin('continue')
+        elif spyder_msg_type == 'set_breakpoints':
+            self.set_spyder_breakpoints(force=True)
+        else:
+            debug_print("No such spyder message type: %s" % spyder_msg_type)
 
     # ---- Private API (overrode by us) ----------------------------
     def _handle_execute_reply(self, msg):
@@ -214,6 +222,7 @@ class NamepaceBrowserWidget(RichJupyterWidget):
                 self.set_namespace_view_settings()
                 self.refresh_namespacebrowser()
             self._kernel_is_starting = False
+            self.ipyclient.t0 = time.monotonic()
 
         # Handle silent execution of kernel methods
         if info and info.kind == 'silent_exec_method' and not self._hidden:
@@ -229,14 +238,22 @@ class NamepaceBrowserWidget(RichJupyterWidget):
         """
         state = msg['content'].get('execution_state', '')
         msg_type = msg['parent_header'].get('msg_type', '')
-        if state == 'starting' and not self._kernel_is_starting:
+        if state == 'starting':
+            # This is needed to show the time a kernel
+            # has been alive in each console.
+            self.ipyclient.t0 = time.monotonic()
+            self.ipyclient.timer.timeout.connect(self.ipyclient.show_time)
+            self.ipyclient.timer.start(1000)
+
             # This handles restarts when the kernel dies
             # unexpectedly
-            self._kernel_is_starting = True
+            if not self._kernel_is_starting:
+                self._kernel_is_starting = True
         elif state == 'idle' and msg_type == 'shutdown_request':
             # This handles restarts asked by the user
             if self.namespacebrowser is not None:
                 self.set_namespace_view_settings()
                 self.refresh_namespacebrowser()
+            self.ipyclient.t0 = time.monotonic()
         else:
             super(NamepaceBrowserWidget, self)._handle_status(msg)
