@@ -62,6 +62,7 @@ class ShellWidget(NamepaceBrowserWidget, HelpWidget, DebuggingWidget):
         # To override the Qt widget used by RichJupyterWidget
         self.custom_control = ControlWidget
         self.custom_page_control = PageControlWidget
+        self.custom_edit = True
         super(ShellWidget, self).__init__(*args, **kw)
 
         self.ipyclient = ipyclient
@@ -250,6 +251,7 @@ the sympy module (e.g. plot)
             # Update checkbox based on user interaction
             CONF.set('ipython_console', 'show_reset_namespace_warning',
                      not box.is_checked())
+            self.ipyclient.reset_warning = not box.is_checked()
 
             if answer != QMessageBox.Yes:
                 return
